@@ -102,13 +102,15 @@ or        L<https://rt.cpan.org/Public/Dist/Display.html?Queue=Cpanel-JSON-XS>
 
 B<Changes to JSON::XS>
 
+- stricter decode_json() as documented. non-refs are disallowed. decode() honors now allow_nonref.
+
 - fixed encode of numbers for dual-vars. Different string representations
   are preserved, but numbers with temporary strings which represent the same number
   are here treated as numbers, not strings. Cpanel::JSON::XS is a bit slower, but
   preserves numeric types better.
 
 - different handling of inf/nan. Default now to null, optionally with -DSTRINGIFY_INFNAN
-  to "inf"/"nan".
+  to "inf"/"nan". [#28, #32]
 
 - added C<binary> extension, non-JSON and non JSON parsable, allows
   C<\xNN> and C<\NNN> sequences.
@@ -116,11 +118,20 @@ B<Changes to JSON::XS>
 - 5.6.2 support; sacrificing some utf8 features (assuming bytes all-over),
   no multi-byte unicode characters.
 
-- interop for true/false overloading. JSON::XS and JSON::PP representations
-  are accepted and JSON::XS accepts Cpanel::JSON::XS booleans [#13]
+- interop for true/false overloading. JSON::XS, JSON::PP and Mojo::JSON 
+  representations for booleans are accepted and JSON::XS accepts
+  Cpanel::JSON::XS booleans [#13, #37]
+  Fixed overloading of booleans. Cpanel::JSON::XS::true stringifies now
+  to true, not 1.
+
+- native boolean mapping of yes and no to true and false, as in YAML::XS.
+  In perl C<!0> is yes, C<!1> is no.
+  The JSON value true maps to 1, false maps to 0. [#39]
 
 - ithread support. Cpanel::JSON::XS is thread-safe, JSON::XS not
 
+- is_bool can be called as method, JSON::XS::is_bool not.
+
 - performance optimizations for threaded Perls
 
 - additional fixes for:
@@ -129,7 +140,10 @@ B<Changes to JSON::XS>
 
   - #10 unshare_hek crash
 
-  - #7 avoid re-blessing where possible (e.g. SvREADONLY restricted hashes)
+  - #7, #29 avoid re-blessing where possible. It fails in JSON::XS for
+   READONLY values, i.e. restricted hashes.
+
+  - #41 overloading of booleans, use the object not the reference.
 
 - public maintenance and bugtracker
 
@@ -145,7 +159,7 @@ B<Changes to JSON::XS>
 
 package Cpanel::JSON::XS;
 
-our $VERSION = '3.0114';
+our $VERSION = '3.0201';
 our @ISA = qw(Exporter);
 
 our @EXPORT = qw(encode_json decode_json to_json from_json);
@@ -191,15 +205,19 @@ Except being faster.
 
 =item $perl_scalar = decode_json $json_text
 
-The opposite of C<encode_json>: expects an UTF-8 (binary) string and tries
-to parse that as an UTF-8 encoded JSON text, returning the resulting
+The opposite of C<encode_json>: expects an UTF-8 (binary) string of an json reference
+and tries to parse that as an UTF-8 encoded JSON text, returning the resulting
 reference. Croaks on error.
 
 This function call is functionally identical to:
 
    $perl_scalar = Cpanel::JSON::XS->new->utf8->decode ($json_text)
 
-Except being faster.
+except being faster.
+
+Note that older decode_json versions in Cpanel::JSON::XS older than
+3.0116 and JSON::XS did not set allow_nonref but allowed them due to a
+bug in the decoder.
 
 =item $is_boolean = Cpanel::JSON::XS::is_bool $scalar
 
@@ -1158,11 +1176,16 @@ up to but not including the least significant bit.
 
 =item true, false
 
-These JSON atoms become C<Cpanel::JSON::XS::true> and C<Cpanel::JSON::XS::false>,
-respectively. They are overloaded to act almost exactly like the numbers
-C<1> and C<0>. You can check whether a scalar is a JSON boolean by using
+These JSON atoms become C<Cpanel::JSON::XS::true> and
+C<Cpanel::JSON::XS::false>, respectively. They are C<JSON::PP::Boolean>
+objects and are overloaded to act almost exactly like the numbers C<1>
+and C<0>. You can check whether a scalar is a JSON boolean by using
 the C<Cpanel::JSON::XS::is_bool> function.
 
+The other round, from perl to JSON, C<!0> which is represented as
+C<yes> becomes C<true>, and C<!1> which is represented as
+C<no> becomes C<false>.
+
 =item null
 
 A JSON null atom becomes C<undef> in Perl.
@@ -1213,22 +1236,30 @@ Perl array references become JSON arrays.
 
 Other unblessed references are generally not allowed and will cause an
 exception to be thrown, except for references to the integers C<0> and
-C<1>, which get turned into C<false> and C<true> atoms in JSON. You can
-also use C<Cpanel::JSON::XS::false> and C<Cpanel::JSON::XS::true> to improve
-readability.
+C<1>, which get turned into C<false> and C<true> atoms in JSON.
+With the option C<allow_unknown>, you can ignore the exception and return
+C<null> instead.
 
-   encode_json [\0, Cpanel::JSON::XS::true]      # yields [false,true]
+   encode_json [\"x"]        # => cannot encode reference to scalar 'SCALAR(0x..)'
+                             # unless the scalar is 0 or 1
+   encode_json [\0, \1]      # yields [false,true]
+
+   allow_unknown->encode_json [\"x"] # yields null
 
 =item Cpanel::JSON::XS::true, Cpanel::JSON::XS::false
 
 These special values become JSON true and JSON false values,
-respectively. You can also use C<\1> and C<\0> directly if you want.
+respectively. You can also use C<\1> and C<\0> or C<!0> and C<!1>
+directly if you want.
+
+   encode_json [Cpanel::JSON::XS::true, Cpanel::JSON::XS::true]      # yields [false,true]
+   encode_json [!1, !0]      # yields [false,true]
 
 =item blessed objects
 
 Blessed objects are not directly representable in JSON, but
-C<Cpanel::JSON::XS> allows various ways of handling objects. See
-L<OBJECT SERIALISATION>, below, for details.
+C<Cpanel::JSON::XS> allows various optional ways of handling
+objects. See L<OBJECT SERIALISATION>, below, for details.
 
 See the C<allow_blessed> and C<convert_blessed> methods on various options on
 how to deal with this: basically, you can choose between throwing an
@@ -1238,9 +1269,10 @@ your own serialiser method.
 =item simple scalars
 
 Simple Perl scalars (any scalar that is not a reference) are the most
-difficult objects to encode: Cpanel::JSON::XS will encode undefined scalars or inf/nan
-as JSON C<null> values, scalars that have last been used in a string context
-before encoding as JSON strings, and anything else as number value:
+difficult objects to encode: Cpanel::JSON::XS will encode undefined
+scalars or inf/nan as JSON C<null> values, scalars that have last been
+used in a string context before encoding as JSON strings, and anything
+else as number value:
 
    # dump as number
    encode_json [2]                      # yields [2]
@@ -1648,7 +1680,7 @@ comparison table for that case.
 For updated graphs see L<https://github.com/Sereal/Sereal/wiki/Sereal-Comparison-Graphs>
 
 
-=head1 INTEROP with JSON and JSON::XS
+=head1 INTEROP with JSON and JSON::XS and other JSON modules
 
 JSON-XS-3.01 broke interoperability with JSON-2.90 with booleans. See L<JSON>.
 
@@ -1656,15 +1688,53 @@ Cpanel::JSON::XS needs to know the JSON and JSON::XS versions to be able work
 with those objects, especially when encoding a booleans like C<{"is_true":true}>.
 So you need to load these modules before.
 
-true/false overloading is supported.
+true/false overloading and boolean representions are supported.
 
 JSON::XS and JSON::PP representations are accepted and older JSON::XS accepts
-Cpanel::JSON::XS booleans.
+Cpanel::JSON::XS booleans. All JSON modules JSON, JSON, PP, JSON::XS, Cpanel::JSON::XS
+produce JSON::PP::Boolean objects, just Mojo and JSON::YAJL not.
+Mojo produces Mojo::JSON::_Bool and JSON::YAJL::Parser just an unblessed IV.
+
+Cpanel::JSON::XS accepts JSON::PP::Boolean and Mojo::JSON::_Bool objects as booleans.
 
 I cannot think of any reason to still use JSON::XS anymore.
 
 =head1 SECURITY CONSIDERATIONS
 
+JSON::XS is not only fast, JSON is generally the most secure
+serializing format, because it is the only one besides
+Data::MessagePack, which does not deserialize objects per default. For
+all languages, not just perl.  The binary variant BSON (MongoDB) does
+more but is unsafe.
+
+It is trivial for any attacker to create such serialized objects in
+JSON and trick perl into expanding them, thereby triggering certain
+methods. Watch L<https://www.youtube.com/watch?v=Gzx6KlqiIZE> for an
+exploit demo for "CVE-2015-1592 SixApart MovableType Storable Perl
+Code Execution" for a deserializer which expands
+objects. Deserializing even coderefs (methods, functions) or external
+data would be considered the most dangerous.
+
+Security relevant overview of serializers regarding deserializing
+objects by default:
+
+                      Objects   Coderefs  External Data
+
+    Data::Dumper      YES       YES       YES
+    Storable          YES       NO (def)  NO
+    Sereal            YES       NO        NO
+    YAML              YES       NO        NO
+    B::C              YES       YES       YES
+    B::Bytecode       YES       YES       YES
+    BSON              YES       YES       NO
+    JSON::SL          YES       NO        YES
+    JSON              NO (def)  NO        NO
+    Data::MessagePack NO        NO        NO
+    XML               NO        NO        YES
+
+    Pickle            YES       YES       YES
+    PHP Deserialize   YES       NO        NO
+
 When you are using JSON in a protocol, talking to untrusted potentially
 hostile creatures requires relatively few measures.
 
@@ -1733,43 +1803,47 @@ license and the GPL.
 =cut
 
 our ($true, $false);
-if ($INC{'JSON/XS.pm'} and $JSON::XS::VERSION ge "3.00") {
-  $true  = $Types::Serialiser::true; # readonly if loaded by JSON::XS
-  $false = $Types::Serialiser::false;
-} else {
-  $true  = do { bless \(my $dummy = 1), "JSON::XS::Boolean" };
-  $false = do { bless \(my $dummy = 0), "JSON::XS::Boolean" };
+BEGIN {
+  if ($INC{'JSON/XS.pm'}
+      and $INC{'Types/Serialiser.pm'}
+      and $JSON::XS::VERSION ge "3.00") {
+    $true  = $Types::Serialiser::true; # readonly if loaded by JSON::XS
+    $false = $Types::Serialiser::false;
+  } else {
+    $true  = do { bless \(my $dummy = 1), "JSON::PP::Boolean" };
+    $false = do { bless \(my $dummy = 0), "JSON::PP::Boolean" };
+  }
 }
 
 sub true()  { $true  }
 sub false() { $false }
-
 sub is_bool($) {
-  UNIVERSAL::isa($_[0], "JSON::XS::Boolean")
-    or UNIVERSAL::isa($_[0], "JSON::PP::Boolean");
+  shift if @_ == 2; # as method call
+  (ref($_[0]) and $_[0]->isa(JSON::PP::Boolean::))
+  or (exists $INC{'Types/Serializer.pm'} and Types::Serialiser::is_bool($_0))
 }
 
 XSLoader::load 'Cpanel::JSON::XS', $VERSION;
 
-package
-  JSON::XS::Boolean;
+{
+  package JSON::PP::Boolean;
 
-use overload
+  use overload
    "0+"     => sub { ${$_[0]} },
    "++"     => sub { $_[0] = ${$_[0]} + 1 },
    "--"     => sub { $_[0] = ${$_[0]} - 1 },
-  # '""'    => sub { ${$_[0]} == 1 ? 'true' : 'false' },
-  #'eq'      => sub {
-  #  my ($obj, $op) = ref ($_[0]) ? ($_[0], $_[1]) : ($_[1], $_[0]);
-  #  if ($op eq 'true' or $op eq 'false') {
-  #    return "$obj" eq 'true' ? 'true' eq $op : 'false' eq $op;
-  #  }
-  #  else {
-  #    return $obj ? 1 == $op : 0 == $op;
-  #  }
-  # },
-   fallback => 1;
-
+   '""'     => sub { ${$_[0]} == 1 ? 'true' : '0' }, # GH 29
+   'eq'     => sub {
+     my ($obj, $op) = ref ($_[0]) ? ($_[0], $_[1]) : ($_[1], $_[0]);
+     if ($op eq 'true' or $op eq 'false') {
+       return "$obj" eq 'true' ? 'true' eq $op : 'false' eq $op;
+     }
+     else {
+       return $obj ? 1 == $op : 0 == $op;
+     }
+   },
+  fallback => 1;
+}
 1;
 
 =head1 SEE ALSO
@@ -1781,13 +1855,13 @@ L<JSON::SL>, L<JSON::DWIW>, L<JSON::YAJL>, L<https://metacpan.org/search?q=JSON>
 
 =head1 AUTHOR
 
-  Marc Lehmann <schmorp@schmorp.de>, http://home.schmorp.de/
+Marc Lehmann <schmorp@schmorp.de>, http://home.schmorp.de/
 
-  cPanel Inc. <cpan@cpanel.net>
+Reini Urban <rurban@cpanel.net>, http://cpanel.net/
 
 =head1 MAINTAINER
 
-  cPanel Inc. <cpan@cpanel.net>
+Reini Urban <rurban@cpanel.net>
 
 =cut
 
